# Chapter 13 of Introduction to Data Science for Business
#

library("XML")
library("quanteda")

# Note the web location for the speech
sbaLocation <- URLencode( "http://www.historyplace.com/speeches/anthony.htm")

# Read and parse HTML file
doc.html = htmlTreeParse(sbaLocation, useInternal = TRUE)

# Extract all paragraphs (HTML tag is p, starting at
# the root of the document). unlist() flattens the list to
# create a character vector.
sba = unlist(xpathApply(doc.html, '//p', xmlValue))

sba <- sba[3:10] # These are the sections of the speech
head(sba, 1) # Review the first paragraph 

#creating word clouds
sbaCorpus <- corpus(sba) # Convert to a corpus 
sbaCorpus <- corpus_reshape(sbaCorpus, to="sentences")
sbaCorpus

# Now convert the corpus to a document-feature matrix
sbaDFM <- dfm(sbaCorpus, remove_punct=TRUE,    
              remove=stopwords("english"))

sbaDFM

textstat_frequency(sbaDFM)[1:10, ]

textplot_wordcloud(sbaDFM) # Make the word cloud 

#now try sentiment analysis
posFile <- "positive-words.txt"
posWords <- scan(posFile, character(0),sep = "\n")
 
posWords <- posWords[-1:-34] # Remove the header
head(posWords,12)

posDFM<- dfm_match(sbaDFM, posWords)
posDFM

posFreq <- textstat_frequency(posDFM)

# Cut it down to retain words that occur at least once
posFreq <- posFreq[posFreq$frequency>0, ]
posFreq


#now do negative words
negFile <- "negative-words.txt"
negWords <- scan(negFile, character(0),sep = "\n")
negWords <- negWords[-1:-34] # Remove the header

negDFM<- dfm_match(sbaDFM, negWords) # Do the matching (need to recheck, this throws an error for JMS)
negDFM # Report on the size of the sparse matrix
negFreq <- textstat_frequency(negDFM)

# Retain negative words that occur at least once
negFreq <- negFreq[negFreq$frequency>0, ]
dim(negFreq) # How many negative words?
negFreq # Show them
sum(negFreq$frequency) # 13 usages of negative words

#topic modeling – with four topics
library("topicmodels")
sbaTopics <- convert(sbaDFM, to = "topicmodels") 
sbaModel <- LDA(sbaTopics, k=4)  

# Plot beta: The per word per topic probability
par(mfrow=c(2,2)) # Make a four pane plot display
for (i in 1:4) {
  topicWords <- exp(sbaModel@beta[i, ])
  names(topicWords) <- sbaModel@terms
  barplot(sort(topicWords,decreasing=TRUE)[1:5],las=2)
}
par(mfrow=c(1,1)) # Restore a full pane plot display

# Review one document that best fits each topic
for (i in 1:4) {
  bestDoc <- which.max(sbaModel@gamma[ ,i])
  print(sbaCorpus[bestDoc])
}

# Distribution of gamma for topic four
hist(sbaModel@gamma[ ,4]) 

# Show the sentences that fit topic four best
sbaCorpus[sbaModel@gamma[ ,4] > 0.75] 


###########################
# case
###########################
 
library("quanteda")
library("topicmodels")
library("readr") 
library("jsonlite")
library("tidyverse")

#read in the full survey 
mydata.list <- jsonlite::fromJSON("completeSurvey.json")

surveyDF <- data.frame(mydata.list)

# just keep surveys with free form text
textSurvey <- surveyDF %>% filter(!is.na(freeText))

# Create a short document title for future reference
textSurvey$docTitle <- paste(textSurvey$Partner.Code,
                             textSurvey$Destination.City,
                             textSurvey$Day.of.Month,sep="_")
textSurvey$docTitle <- paste(textSurvey$Partner.Code,
                             textSurvey$Destination.City,
                             textSurvey$Day.of.Month,
                             c(1:nrow(textSurvey)),
                             sep="_")

#first just focus on detractors
detractDF <- textSurvey %>% 
  filter(Likelihood.to.recommend < 7)

# Convert to a corpus
flightCorpus <- corpus(detractDF$freeText,
                       docnames= detractDF$docTitle) 

# create a new set of stopwords  to include
# southeast and flight as well the word 's'  
myStop <- c(stopwords("english"), 
            's', 'southeast', 'flight')

# Now convert the corpus to a document-feature matrix
flightDFM <- dfm(flightCorpus, 
                 remove_punct=TRUE, 
                 remove=myStop)
flightDFM  

#most frequent words used by detractors
wordDF <- flightDFM %>% 
  textstat_frequency(n = 50) %>%
  data.frame() %>%
  select(word=feature, freq=frequency)

wordDF %>% 
  filter(freq>20) %>%
  ggplot(aes(x = reorder(word, freq), y = freq)) +
  geom_point() +
  coord_flip() +
  labs(x = NULL, y = "Frequency") +
  theme_minimal() + ggtitle("frequent words (DETRACTOR)")

set.seed(11)
textplot_wordcloud(flightDFM) 


#topic modeling

#start with all the free form text
library("topicmodels")
set.seed(200)
flightCorpus <- corpus(textSurvey$freeText, 
                       docnames = textSurvey$docTitle)

# Now convert the corpus to a document-feature matrix
flightDFM <- dfm(flightCorpus, 
                 remove_punct=TRUE, remove=myStop)

# Convert the DFM to the topic model class
flightTopics <- convert(flightDFM, to = "topicmodels")

# Create the topic model, while controlling 
# randomization for a consistent result
flightModel <- LDA(flightTopics, 
                   control = list(alpha = 0.1, 
                                  seed=c(1:10), 
                                  nstart=10), k=7) 

# Add Likelihood.to.recommend to the gamma matrix
predMatrix <- cbind(textSurvey$Likelihood.to.recommend,
                    flightModel@gamma)

# Convert to a data frame
predDF <- as.data.frame(predMatrix)

# Change the column labels to something more helpful
colnames(predDF) <- c("LTR", "Topic1", "Topic2", 
                      "Topic3","Topic4", "Topic5", "Topic6",
                      "Topic7")

library(caret) 
model.lm <- train(LTR ~ Topic1 + Topic2 + Topic3 +
                    Topic4 + Topic5 + Topic6,
                  method = "lm", 
                  data = predDF)

summary(model.lm)

#Create a function to generate the barchart
createTextBarchart <- function(topicNum) {
  topicDF <- data.frame(importance=
                          exp(flightModel@beta[topicNum, ]), 
                        words=flightModel@terms)
  
  #sort / keep the top 10 rows (based on importance)
  topWordDF <- arrange(topicDF, desc(importance)) %>% 
    slice(1:10)
  
  #display the most important words in this topic
  viz <- ggplot(topWordDF) +
    aes(x=reorder(words,-importance), y=importance) +
    geom_col() + 
    xlab(paste0("Words in Topic", topicNum))+
    theme(axis.text.x = 
            element_text(angle = 90, hjust = 1))
  return(viz)
}

vizTopic2 <- createTextBarchart(2)

vizTopic5 <- createTextBarchart(5)

library(gridExtra)
grid.arrange(vizTopic2, vizTopic5, nrow=2,
             top = "Exploring Important Topics")

#explore the topics
topic2Comments <- flightModel@gamma[ ,2]> .99
sum(topic2Comments)

flightCorpus[topic2Comments] [10]                                                     

topic5Comments <- flightModel@gamma[ ,5]> .99
sum(topic5Comments)
flightCorpus[topic5Comments] [10]                                                     


#initial the positive and negative counts
textSurvey$freeTextNegCnt <- 0
textSurvey$freeTextPosCnt <- 0

#loop through each free form text survey response
for (i in 1:nrow(textSurvey)) {
  # Convert the response to a corpus 
  flightCorpus <- corpus(textSurvey$freeText[i], 
                         docnames = textSurvey$docTitle[i]) 
  
  # Convert the corpus to a document-feature matrix 
  flightDFM <- dfm(flightCorpus, 
                   remove_punct=TRUE, remove=myStop)
  
  #count negative words
  negDFM <- dfm_match(flightDFM, negWords)
  if(!is_empty(negDFM@x)) {
    negFreq <- textstat_frequency(negDFM)
    negFreq <- negFreq[negFreq$frequency>0, ]
    textSurvey$freeTextNegCnt[i] <- 
      sum(negFreq$frequency) 
  } 
  
  #count positive words  
  posDFM <- dfm_match(flightDFM, posWords)
  if(!is_empty(posDFM@x)) {
    posFreq <- textstat_frequency(posDFM)
    posFreq <- posFreq[posFreq$frequency>0, ]
    textSurvey$freeTextPosCnt[i] <- 
      sum(posFreq$frequency) 
  }  
} 

textSurvey$sentiment <- 
  textSurvey$freeTextPosCnt - textSurvey$freeTextNegCnt

textSurvey$detractor <- as.factor(textSurvey$Likelihood.to.recommend<7)

textSurvey %>% 
  group_by(detractor) %>%
  summarize(mean(sentiment))  







