# Chapter 14 of Introduction to Data Science for Business
#
# In chapter 14, we need separate files for each version
# of the app that the chapter describes.
#
# This is the case study app

library(shiny)
library(tidyverse)
library(ggplot2)

ui <- fluidPage(
  selectInput("attribute", "Attribute:",
              c("Age" = "roundedAge",
                "Origin State" = "Origin.State",   
                "Destination State" = "Destination.State",   
                "Partner Name" = "Partner.Name",
                "Arrive Delay" =  "roundedArrivalDelay",
                "Big Arrival Delay" = "Big.Arrival.Delay",
                "Class of Travel" = "Class",
                "Type of Travel" = "Type.of.Travel",
                "Loyalty" = "roundedLoyalty",
                "Price Sensitivity" = "Price.Sensitivity",
                "Airline Status" = "Airline.Status",
                "Shopping Amount at Airport" = 
                  "roundedShopping.Amount.at.Airport",
                "Eating and Drinking at Airport" = 
                  "roundedEating.and.Drinking.at.Airport"
              )),
  plotOutput("plot")
)

# CalcNPS function - calculate a net promoter score
# ltr - a vector of likliehood-to-recommend values
# neutral - the range, below which someone
# is considered a detractor and above which
# some is consider a promoter (default is 7 to 8)
CalcNPS <- function(ltr, neutral=c(7,8)) {
  
  # create a new vector with just the promoters
  #  then calculate the length of the promoters vector
  promoters <- ltr[ltr > max(neutral)]
  numPromoters <- length(promoters)
  
  # calculate the number of detractors 
  detractorsTrueFalse <- ltr < min(neutral)
  numDetractors <- sum(detractorsTrueFalse)
  
  # calculate NPS, based on the length of ltr 
  #  and the number of promotors and detractors
  total <- length(ltr)
  nps <- (numPromoters/total - 
            numDetractors/total)*100
  
  #return the NPS value
  return(nps)
}

#generate a barchart, with fill color representing the number of surveys
#and the user defining the attribute to be charted
server <- function(input, output) {
  survey <- readSurveyData()
  output$plot <- renderPlot(
    survey %>% 
      group_by_at(input$attribute) %>%
      summarize(
        nps = CalcNPS(Likelihood.to.recommend), 
        numSurveys = n() ) %>%
      ggplot() +
      aes_string(x=input$attribute) +
      aes(y=nps, fill=numSurveys) +
      geom_col() + 
      theme(text = element_text(size=16),
            axis.text.x = element_text(angle = 90, 
                                       hjust = 1))
  )
}

#read in the json survey, similar to other chapters
readSurveyData <- function() {
  library(jsonlite)
  mydata.list <- fromJSON("completeSurvey.json" )
  surveyWithNA <- data.frame(mydata.list)
  
  #Remove rows with‘Likelihood.to.recommend’ == NA
  survey <- surveyWithNA %>% 
    filter(!is.na(Likelihood.to.recommend))
  
  #Was the arrival was delayed a lot
  survey$Big.Arrival.Delay <- 
    survey$Arrival.Delay.in.Minutes > 5
  
  survey$roundedAge <- round(survey$Age, -1)
  
  roundedDelay <- survey$Arrival.Delay.in.Minutes
  roundedDelay [roundedDelay > 300] <- 300
  survey$roundedArrivalDelay <- round(roundedDelay, -1) 
  
  survey$roundedLoyalty <- round(survey$Loyalty, 1)
  survey$roundedShopping.Amount.at.Airport <- 
    round(survey$Shopping.Amount.at.Airport, -1)
  survey$roundedEating.and.Drinking.at.Airport <- 
    round(survey$Eating.and.Drinking.at.Airport, -1)
  
  return(survey)
}

# Run the application 
shinyApp(ui = ui, server = server)



