# Chapter 15 of Introduction to Data Science for Business
#

# Keras provides an R interface
# to the Python deep learning package Keras
#
install.packages("keras")

#install.packages("devtools")
#library(devtools)
#install_github("rstudio/keras")

# Keras uses Tensorflow at backend to install
# both Keras and Tensorflow run following the
# following commands.
library(keras)
install_keras()

library(tidyverse)
library(caret)

df <- read_csv("testData.csv")
head(df, 5)

#create test and train datasets
trainList <-
  createDataPartition(y=df$Y, p=.80, list=FALSE)
trainData <- df[trainList,]
testData <- df[-trainList,]

# input_shape is the number of X inputs (we have 3 inputs)
# units is the number of neurons
# ReLU (rectified linear unit) is the typical activation
# function for intermediate layers,
# linear (identity) for positive values, and zero for negative

# sigmoid is used in the final layer for binary analysis
model = keras_model_sequential() %>%
  layer_dense(input_shape=3, units=64, activation="relu") %>%
  layer_dense(units=1, activation = "sigmoid")

# loss == how to measure error.
# Use categorical crossentropy in classification problems
# where only one result can be correct.
# optimizer == how to optimize during iterations
# optimizer_rmsprop is a good default
# metrics == List of metrics to be evaluated by the model
# often use metrics='accuracy’
compile (model,
         loss = "binary_crossentropy",
         optimizer = optimizer_rmsprop(),
         metrics = "accuracy")

#pull the columns from the dataframe
#these are all functionally equivalent
x_data <- as.matrix(trainData[,1:3])
x_data <- as.matrix(trainData[c('X1', 'X2', 'X3')])
x_data <- trainData %>% select (c(X1, X2, X3)) %>% as.matrix()

# epochs: # times that the learning algorithm will work
# through the entire training dataset.
# batch_size: the number of samples to work through before
# updating the internal model parameters.
# --> 64, 128, 256 are typical default numbers
# validation_split: The % to validate the model
# (the rest is to train the model)
history = fit(model,
              x_data,
              trainData$Y,
              epochs = 20,
              batch_size = 128,
              validation_split = 0.2)

x_data_test <- as.matrix(testData[,1:3])
y_data_pred=predict_classes(model, x_data_test)

#see how good the prediction was
confusionMatrix(as.factor(y_data_pred), as.factor(testData$Y))


#run the SVM
trainData$Y <- as.factor(trainData$Y)
svm.model <- train(Y ~ X1+X2+X3, data = trainData,
                   method = "svmRadial",
                   trControl=trainControl(method = "none"),
                   preProcess = c("center", "scale"))

#use the model on the test data
testData$Y <- as.factor(testData$Y)
predictValues <- predict(svm.model, newdata=testData)
confusionMatrix(predictValues, testData$Y)


#image analysis

library(tidyverse)
library(keras)
library(caret)

# Load the images of digits: By default this will
# get the data from a Google repository. If you prefer
# to use a local version of the file, you will have to
# supply a full pathname.
mnist <- dataset_mnist()

# Note that on some machines, the 'dataset_mnist()' does not work.
# in this situation, you can use the following code
#load the images of digits:
#    https://storage.googleapis.com/tensorflow/tf-keras-datasets/mnist.npz
#      note: if loading a local file, need full path
mnist <- dataset_mnist(path = "/Users/jsaltz/GoogleDrive-SYR/NewBook/DeepLearning Data/mnist.npz")


#look at an images (only 28x28) and the known value
img <- mnist$train$x[400,,]
image(1:28, 1:28, img, col = gray.colors(start=0, end=1, n=256))
#Output the training value
mnist$train$y[400]

model <- keras_model_sequential() %>%
  # Flatten the square 28*28 image
  layer_flatten(input_shape=c(28, 28)) %>%
  #After the pixels are flattened, the network consists of
  #two layers. Starting with 128 units (neurons)
  layer_dense(units = 128, activation = "relu") %>%
  #The final layer is a 10-node softmax layer
  layer_dense(units = 10, activation = 'softmax')

#same code as previous example to compile the model, except
# we use ‘categorical_crossentropy’, since we have 10 choices
# (not just two choices)
model %>% compile(
  loss = 'categorical_crossentropy',
optimizer = optimizer_rmsprop(),
metrics = c('accuracy')
)

#create a matrix of choices
y_train <- to_categorical(mnist$train$y, 10)

#same code as previous example to run the model
history <- model %>% fit(
  mnist$train$x, y_train,
  epochs = 30, batch_size = 128,
  validation_split = 0.2
)


# Add a third layer of nodes (neurons)
# And add dropout layers-sets % of neuron weights to 0
# randomly)
model <- keras_model_sequential() %>%
  layer_flatten(input_shape=c(28, 28)) %>%
  layer_dense(units = 256, activation = 'relu') %>%
  layer_dropout(rate = 0.4) %>%
  layer_dense(units = 128, activation = 'relu') %>%
  layer_dropout(rate = 0.3) %>%
  layer_dense(units = 10, activation = 'softmax')

#same code as previous example to run the model
model %>% compile(
  loss = 'categorical_crossentropy',
optimizer = optimizer_rmsprop(),
metrics = c('accuracy')
)

#same code as previous example to run the model
history <- model %>% fit(
   mnist$train$x, y_train,
   epochs = 30, batch_size = 128,
   validation_split = 0.2
   )

# Explore model accuracy
library(caret)
pred.digit <- model %>% predict_classes(mnist$test$x)
confusionMatrix(as.factor(pred.digit), as.factor(mnist$test$y))

which(pred.digit != mnist$test$y)

pred.digit[9983]

mnist$test$y[9983]

img <- mnist$test$x[9983,,]
image(1:28, 1:28, img, col = gray.colors(start=0, end=1, n=256))

library(e1071)
#convert the input data (X image data) into something usable
x_train <- array_reshape(mnist$train$x,
                         c(nrow(mnist$train$x), 784))
trainData <- as.data.frame(x_train)

#add the digit value to the dataset
trainData <- cbind(trainData, digit=mnist$train$y)
trainData$digit <- as.factor(trainData$digit)

#run the SVM - takes about 20-90 minutes on a high end laptop
svm.model <- train(digit ~., data = trainData,
                   method = "svmRadial",
                   trControl=trainControl(method = "none"),
                   preProcess = c("center", "scale"))


#using a pre-build model
# After training completes, you can save your model
model %>% save_model_hdf5("my_model.h5")

# Call load_model_hdf5 to load the model
new_model <- load_model_hdf5("my_model.h5")

# Call load_model_hdf5 to load the model
# (once you have saved the file locally)
model <- load_model_hdf5("xception.h5")

model %>% compile(
  loss = 'categorical_crossentropy',
  optimizer = optimizer_rmsprop(),
  metrics = c('accuracy')
)


# be able to decode predictions to animal names
library(jsonlite)
imageNames <- fromJSON("imagenet_class_index.json")

imageNamesDF <- as.data.frame(imageNames,
  stringsAsFactors = FALSE)

predictedImageNames <- as.character(imageNamesDF[2,1:1000])
predictedImageNames[1:3]

#use the model on a test image (of a cat)
testImage <- "cat.jpg"

# first just display image using magick package
#install.packages("magick")
library(magick)
#im <- image_read(imageName)
im <- image_read(testImage)

plot(im)

# Next load image, convert to array and reshape for our model
img <- image_load(testImage, target_size = c(299,299))
x <- image_to_array(img)
x <- array_reshape(x, c(1, dim(x)))
# xception_preprocess_input() is used for image preprocessing
x <- xception_preprocess_input(x)
# predict the type of image
predictions <- model %>% predict(x)

# create a dataframe of prediction % and the name of that animal
#predsDF <- data.frame(predPercent=predictions,
#                      predictedImageNames)

predsDF <- data.frame(predPercent=predictions[1,], 
                      predictedImageNames)


# sort the predictions
predsDF <- predsDF %>% arrange(desc(predPercent))

# show the predictions
predsDF[1:5,]


#############################
# case study
#############################
#load survey
library(jsonlite)
mydata.list <- fromJSON("completeSurvey.json")
surveyWithNA <- data.frame(mydata.list)
#Remove rows with‘Likelihood.to.recommend’ == NA
survey <- surveyWithNA %>%
  filter(!is.na(Likelihood.to.recommend))

#look for detractors – 1 is a detractor
survey$Detractor <- survey$Likelihood.to.recommend <7
survey$Detractor <- as.integer(survey$Detractor)

#need to add this code
survey$Type.of.Travel <- as.factor(survey$Type.of.Travel)
survey$Airline.Status <- as.factor(survey$Airline.Status)

survey$Type.of.Travel <- as.integer(survey$Type.of.Travel)
survey$Airline.Status <- as.integer(survey$Airline.Status)


trainList <-
  createDataPartition(y=survey$Detractor,p=.80,list=FALSE)
trainData <- survey[trainList,]
testData <- survey[-trainList,]

x_data <- as.matrix(
  trainData[,c("Type.of.Travel", "Airline.Status")])

# Define the model
model <- keras_model_sequential() %>%
  layer_dense(units = 256, activation = 'relu',
              input_shape = ncol(x_data)) %>%
  layer_dropout(rate = 0.4) %>%
  layer_dense(units = 128, activation = 'relu') %>%
  layer_dropout(rate = 0.3) %>%
  layer_dense(units = 1, activation = "sigmoid")

# Compile the model
compile(model,
        loss = "binary_crossentropy",
        optimizer = optimizer_rmsprop(),
        metrics = "accuracy")

# Run the model
history = fit(model,
              x_data,
              trainData$Detractor,
              epochs = 20,
              batch_size = 128,
              validation_split = 0.2)

#do prediction
x_data_test <- as.matrix(
  testData[,c("Type.of.Travel", "Airline.Status")])

y_data_pred=predict_classes(model, x_data_test)

#see how good the prediction was
confusionMatrix(as.factor(y_data_pred),
                as.factor(testData$Detractor))

