# Chapter 6 of Introduction to Data Science for Business
#

# This is a repeat of the Chapter 5 code for convenience
#read in the census data set
readCensus <- function(urlToRead) {
  
  #read the data from the web
  testFrame <- read_csv(url(urlToRead))
  #testFrame <- read_csv("nst-est2011-01.csv")

  #use pipes to keep updating testFrame
  testFrame <- testFrame %>%
    
    #only keep the first 5 columns
    select(1:5) %>%
    
    #convert columns to numbers & rename columns
    mutate(april10census=Numberize(testFrame$X2))%>%
    mutate(april10base=Numberize(testFrame$X3)) %>%
    mutate(july10pop=Numberize(testFrame$X4)) %>%
    mutate(july11pop=Numberize(testFrame$X5)) %>%
    
    #remove the old columns that have been updated
    select( -X2, -X3, -X4, -X5) %>%
    
    #remove the first 8 rows (‘header information’)
    slice(-1:-8) %>%
    
    #rename the first column to StateName
    rename(stateName=1) %>%
    
    #remove the last rows (tail info to see)
    slice(c(-52, -54:-n())) %>%
    
    #remove the ‘dot’ from the state name
    mutate(stateName=
             str_replace_all(stateName, "\\.","")) 
  
  #remove the old rownames, which are now confusing
  rownames(testFrame) <- NULL
  
  return(testFrame)
}

# Numberize() - Gets rid of commas and other junk and 
# converts to numbers
# Assumes that the inputVector is a list of data that 
# can be treated as character strings
Numberize <- function(inputVector)
{
  # Get rid of commas and spaces
  inputVector<-str_replace_all(inputVector, "[, ]","")
  
  #convert to a number and then return that number
  return(as.numeric(inputVector))
}

library(tidyverse)
USregionPops <- readCensus(      
  "http://www2.census.gov/programs-surveys/popest/tables/2010-2011/state/totals/nst-est2011-01.csv")

#new code for this chapter
set.seed(1)
sample(USregionPops$april10census, size=8,
       replace=TRUE)

mean(sample(USregionPops$april10census,size=8, 
            replace=TRUE))

#replicating samples
replicate(4, mean(sample(USregionPops$april10census, 
                         size=8,replace=TRUE)),simplify=TRUE)

mean(replicate(400,mean( 
  sample(USregionPops$april10census, size=8, 
         replace = TRUE)), simplify=TRUE))

mean(replicate(4000,  mean( 
  sample(USregionPops$april10census, size=8, 
         replace = TRUE)), simplify=TRUE))

hist(replicate(40000,  mean( 
  sample(USregionPops$april10census, size=8, 
         replace = TRUE)), simplify=TRUE),
          main="Sampling Distribution: 40,000 Means",
          xlab="Means of april10census")

#Law of large numbers & Central Limit Theorum
mean(replicate(100, mean( 
  sample(USregionPops$april10census,size = 52,  
         replace = TRUE)), simplify=TRUE))

mean(replicate(100, mean(
  sample(USregionPops$april10census, size=5200, 
         replace=TRUE)), simplify=TRUE))


set.seed(1)
SampleMeans <- replicate(10000,  
                 mean(sample(USregionPops$april10census,
                             size = 8,  replace = TRUE)), 
                 simplify=TRUE)
length(SampleMeans)
mean(SampleMeans)

summary(SampleMeans)

quantile(SampleMeans, probs=c(0.25,0.50,0.75))

quantile(SampleMeans, probs=c(0.025,0.975))


#comparing two samples
MysterySample <- c(2238603,2662480,1632934,822441,
                   1955577,1084979,637026,3388768)
mean(MysterySample)

quantile(SampleMeans, probs=c(0.005,0.995))

sd(SampleMeans)

sd(USregionPops$april10census)/sqrt(8)

StdError <- sd(USregionPops$april10census)/sqrt(8)
CutPoint975 <- mean(USregionPops$april10census) + 
  (2 * StdError)
CutPoint975


##########################
# case study
##########################
library(tidyverse)

smallSurvey <- read_csv("smallSurvey.csv")
#remove the NAs from the LTR column
smallSurvey <- filter(smallSurvey, !is.na(LTR))

#read in the new survey results after the treatment
newTreatmentDF <- read_csv("newTreatmentSurvey.csv")
nrow(newTreatmentDF)

# Provided here for completeness, also in Chapter 4 code
# CalcNPS function - calculate a net promoter score
# ltr - a vector of likliehood-to-recommend values
# neutral - the range, below which someone
# is considered a detractor and above which
# some is consider a promoter (default is 7 to 8)
CalcNPS <- function(ltr, neutral=c(7,8)) {
  
  #create a new vector with just the promoters
  #then calculate the length of the promoters vector
  promoters <- ltr[ltr > max(neutral)]
  numPromoters <- length(promoters)
  
  #calculate the number of detractors by summing the 
  #  elements that are less than 7
  detractorsTrueFalse <- ltr < min(neutral)
  numDetractors <- sum(detractorsTrueFalse)
  
  #calculate NPS, based on the length of ltr 
  #  and the number of promotors and detractors
  total <- length(ltr)
  nps <- (numPromoters/total - 
            numDetractors/total)*100
  
  #return the NPS value
  return(nps)
}

CalcNPS(smallSurvey$LTR)
CalcNPS(newTreatmentDF$LTR)


#Use the set.seed to ensure consistent results
set.seed(100)

# We sample many times from smallSurvey
# using n=120 to match the number of 
# observations in the treatment survey.
replicatedNPS <- replicate(1000, 
                           CalcNPS(sample(smallSurvey$LTR,
                                          size=120,
                                          replace=TRUE)))

#Calculate the 95% certainty value
thresholdValues <- quantile(replicatedNPS, probs=c(0.025,0.975))

#Generate a histogram of NPS values and the 95% value
hist(replicatedNPS)

# Mark the 95% central region
abline(v=min(thresholdValues))
abline(v=max(thresholdValues))

#show the new NPS value (linetype is a dashed line)
newTreatmentNPS <- CalcNPS(newTreatmentDF$LTR)
abline(v=newTreatmentNPS,  lty="dashed")




