# Chapter 7 of Introduction to Data Science for Business
#

# install.packages("readxl") # Uncomment if you need to install from internet
library("readxl")
library("tidyverse")

#define the URL across multiple lines
# (just to make it easier to cut and paste code)
part1 <-"http://www2.census.gov/programs-surveys/"
part2 <- "popest/tables/2010-2011/state/totals/"
part3 <- "nst-est2011-01.xls"
dataFile <- paste0(part1,part2,part3)

#download the file from the web, into tmpExcelFile
download.file(dataFile, "tmpExcelFile.xls")

#now read the excel file into R
testFrame <- read_excel("tmpExcelFile.xls")

glimpse(testFrame)

testFrame <- slice(testFrame, -1:-8)

tail(testFrame)

testFrame <- slice(testFrame, -54:-58)
testFrame <- slice(testFrame, -52)

tail(testFrame)

testFrame$stateName <- pull(testFrame, 1)

testFrame <- testFrame[,-1]

testFrame$stateName <- 
  str_replace(testFrame$stateName,"\\.","")

testFrame <- testFrame %>% rename(april10census = '...2')
testFrame <- testFrame %>% rename(april10base = '...3')
testFrame <- testFrame %>% rename(july10pop = '...4')
testFrame <- testFrame %>% rename(july11pop = '...5')

testFrame <- testFrame %>% mutate_at( 
  vars(april10census,april10base,july10pop,july11pop), 
  as.numeric)

# Working with data from external databases

# Install MySQL on local computer before starting this:
# Set up the installation for MySQL 5.X compatibility because
# version 3.6.0 of R could not use the more secure password
# system from MySQL 8.X.
#
# In addition, before running the commands below, you need to
# create an initial database in the MySQL system. On Windows
# this can be done from the MySDQL Workbench. On the Mac, you 
# can use a command line like this:
# mysql --user=root --password=MyPW --execute="CREATE DATABASE test"

library("RMySQL")
con <- dbConnect(dbDriver("MySQL"), username="root", password="MyPW", dbname="test")

dbListTables(con)

# Also for MySQL 8, we needed to run this command to allow
# a local table to be passed into MySQL: 
# mysql --user=root --password=MyPW --execute="SET GLOBAL local_infile = true"
dbWriteTable(con, name="census", value=testFrame, overwrite=TRUE)

dbListTables(con)

dbGetQuery(con, "SELECT stateName, july11pop FROM census WHERE july11pop<1000000")

dbDisconnect(con)


#comparing SQL and R/Tidyverse
library("sqldf")
options(sqldf.driver = "SQLite")
sqldf("select avg(april10base) From testFrame")

mean(testFrame$april10base)

sqldf("select stateName From testFrame
  				Where july11pop<1000000")

testFrame %>% 
  filter(july11pop<1000000) %>% 
  select(stateName)

sregions <- data.frame(stateName=state.name, region=state.region)
head(sregions)

newFrame <- sqldf("select testFrame.stateName, july11pop, region from testFrame join sregions on testFrame.stateName=sregions.stateName")

sqldf("select AVG(july11pop) From newFrame 
 						Group by region")

newFrame %>% 
  group_by(region) %>%    
  summarise(regionMean=mean(july11pop))

newFrame %<>%  
  group_by(region) %>%
  mutate(regionMean = mean(july11pop))

newFrame %>% 
  select(stateName, region, regionMean) %>% 
  slice(1)


# Accessing JSON Data 

# Load the necessary libraries before getting started
library("RCurl")
library("jsonlite")

#Format a URL to get a JSON exchange rate
getRateURL <- function(YOUR_APP_ID) {
  rURL <- "https://openexchangerates.org/api/latest"
  secondPart <- ".json?app_id="
  url <- paste0(rURL,secondPart,YOUR_APP_ID)
  return(url)
}

getRateURL("123")


appID <- "123" # Replace with your app_id from the site

#get the URL
url <- getRateURL(appID)

#get the data from the web
apiResult <- getURL(url)

#parse the JSON data into a dataframe
fxStruct <- fromJSON(apiResult)
fxStructDF <- data.frame(fxStruct)

#look at the Euro conversion rate (for USD)
fxStructDF$rates.EUR


# CitiBike JSON example
bikeURL <- paste0('https://gbfs.citibikenyc.com/',
                  'gbfs/en/station_status.json')
apiResult <- getURL(bikeURL)
results <- fromJSON(apiResult)
 
stations <- results$data$stations

#### new code
cols <- c('num_bikes_disabled','num_docks_disabled', 'station_id',
          'num_ebikes_available', 'num_bikes_available', 'num_docks_available')
stations = stations[,cols]

nrow(stations)
str(stations)

mean(stations$num_docks_available)
mean(stations$num_bikes_available) 

bikesAvailDF <- stations[stations$num_bikes_available>0,]
nrow(bikesAvailDF)



##########################
# case study
##########################

#Read in the JSON datafile 

library(jsonlite) 
mydata.list <- fromJSON("completeSurvey.json" )
surveyWithNA <- data.frame(mydata.list)

#Look at the datafile (don’t use View – it's too big)
glimpse(surveyWithNA)

#Remove rows with the ‘Likelihood.to.recommend’ == NA
survey <- surveyWithNA %>% 
  filter(!is.na(Likelihood.to.recommend))

#What percentage was removed via the cleaning?
(1 - nrow(survey)/nrow(surveyWithNA) ) * 100

#Remove any row with an NA
survey <-   na.omit(surveyWithNA)   

#What percentage was removed via this cleaning?
(1 - nrow(survey)/nrow(surveyWithNA) ) * 100

#Revert to removing rows with the ‘Likelihood.to.recommend’ == NA
survey <- surveyWithNA %>% 
  filter(!is.na(Likelihood.to.recommend))

#Was the arrival was delayed by more than 5 minutes.
survey$Big.Arrival.Delay <- 
  survey$Arrival.Delay.in.Minutes > 5

#Was the person a detractor.
survey$Detractor <- survey$Likelihood.to.recommend < 7

survey %>% 
  group_by(Big.Arrival.Delay) %>%
  summarize(percentDetractor = mean(Detractor) )

