# Chapter 8 of Introduction to Data Science for Business
#

# install.packages("readxl") # Uncomment if you need to install from internet
library(readxl)

#See chapter 5 for the function definition
#dfStates <- readCensus("nst-est2011-01.csv")

#See chapter 5 for the function definition - fixed to use the URL
dfStates <- readCensus(      
  "http://www2.census.gov/programs-surveys/popest/tables/2010-2011/state/totals/nst-est2011-01.csv")


#generate a histogram of population by state
hist(dfStates$july11pop)

barplot(dfStates$july11pop, names.arg = 
          dfStates$stateName, las=2)


#Using GGPLOT2

#a function to check about a function
EnsurePackage<-function(x) {
  x <- as.character(x)
  
  if (!require(x,character.only=TRUE)) {

    install.packages(pkgs=x,
                     repos="http://cran.r-project.org")
    require(x,character.only=TRUE)
    }
}

EnsurePackage("tidyverse")


#generate a ggplot histogram
#The data
g <- ggplot(dfStates)

#Add the aesthetic to the plot variable
g <- g + aes(x=july11pop)

#add the geometry
g <- g + geom_histogram(binwidth=5000000, 
                        color="black", fill="white")

#add the title
g <- g + ggtitle("states population histogram")
g

ggplot(dfStates) + aes(y=july11pop) + geom_boxplot()


#create the two new attributes
dfStates$popChange <- dfStates$july11pop -
  dfStates$july10pop

dfStates$increasePop <- 
  ifelse(dfStates$popChange > 0, 
         "positive", "negative")

#now generate / display the boxplot
ggplot(dfStates) +
  aes(x=increasePop, y=july11pop) +
  geom_boxplot() +
  ggtitle('Population by type of change')



#generate a sorted linechart
ggplot(dfStates) +
  aes(x=reorder(stateName, july11pop),  
      y=july11pop, group=1) +
  geom_line() +
  theme(axis.text.x = element_text(angle = 90,  
                                   hjust = 1))

#generate a sorted barchart
ggplot(dfStates) +
  aes(x=reorder(stateName, july11pop),  
      y=july11pop) +
  geom_col() +
  theme(axis.text.x = element_text(angle = 90,  
                                   hjust = 1))


#more advanced viz

#create the % change attribute
dfStates$percentChange <- 
  dfStates$popChange/dfStates$july10pop * 100

#visualize the % change
ggplot(dfStates) +
  aes(x=reorder(stateName, july11pop),  
      y=july11pop, group=1, fill=percentChange) +
  geom_col() +
  theme(axis.text.x = element_text(angle = 90, 
                                   hjust = 1))


#a ggplot scatter chart
g <- ggplot(dfStates)+ 
  aes(x=popChange, y=percentChange,
      size=july11pop, color=july11pop) +
  geom_point()
g  

g + geom_text(size=3, vjust=2) + aes(label=stateName)


#a more advanced scatter plot
#define the criteria to show the text for the state
minPerChange <- 1
minPopChange <- 100000

#determine the key states
dfStates$keyState <- dfStates$popChange>minPopChange & 
  dfStates$percentChange > minPerChange

#create a dataframe of just the key states
keyStatesDF <- dfStates[dfStates$keyState,]

#define labels showing min, max and median
minLabel <- format(min(dfStates$july11pop), 
                   big.mark=",", trim=TRUE)
maxLabel <- format(max(dfStates$july11pop), 
                   big.mark=",", trim=TRUE)
medianLabel <- format(median(dfStates$july11pop), 
                      big.mark=",", trim=TRUE) 

#now we can generate the plot
g <- ggplot(dfStates) +
  aes(x=popChange,y=percentChange,
      size=july11pop, color=july11pop, 
      shape=keyState) +
  geom_point() +
  geom_text(data=keyStatesDF,  
            aes(label=stateName, hjust=1, vjust=-1)) 

#add the color scale
g + scale_color_continuous(name="Pop", 
                           low = "white",high = "black",                                    
                           breaks = with(dfStates, 
                                         c(min(july11pop), median(july11pop), 
                                           max(july11pop))), 
                           labels = c(minLabel, medianLabel, maxLabel)) 




#################################
# case study
###########################
library(jsonlite) 
readSurveyData <- function(fileName) {
  mydata.list <- fromJSON(fileName)
  surveyWithNA <- data.frame(mydata.list)
  
  #Remove rows with‘Likelihood.to.recommend’ == NA
  survey <- surveyWithNA %>% 
    filter(!is.na(Likelihood.to.recommend))
  
  #Was the arrival was delayed a lot
  survey$Big.Arrival.Delay <- 
    survey$Arrival.Delay.in.Minutes > 5
  
  #Was the person a detractor.
  survey$Detractor <- 
    survey$Likelihood.to.recommend < 7
  
  return(survey)
}

#read in the survey data
survey <- readSurveyData("completeSurvey.json")

#generate a histogram of Likelihood.to.recommend
survey %>% 
  ggplot() + 
  geom_histogram(binwidth=1, 
                 fill="black", color="white",
                 aes(x= Likelihood.to.recommend))  

survey %>% 
  ggplot() + 
  geom_histogram(binwidth=1, 
                 aes(x= Likelihood.to.recommend)) +  
  facet_wrap(~Type.of.Travel) 	 


#create a simple dataframe to visualize 
summaryDF <- survey %>% 
  group_by(Big.Arrival.Delay) %>%
  summarize( percentDetractor = mean(Detractor) )

#generate the barchart 
ggplot(summaryDF) +
  aes(x=Big.Arrival.Delay, y=percentDetractor) +
  geom_col() 

#explore detractors when there is a big delay
survey %>% 
  group_by(Big.Arrival.Delay) %>%
  summarize(percentDetractor = mean(Detractor) ) %>%
  ggplot() +
    aes(x=Big.Arrival.Delay, y=percentDetractor) + 
    geom_col() 


#round age - and make it a factor
survey$roundAge <- 
  as.factor(round(survey$Age, digits=-1))

#generate the box plot
ggplot(survey)+
  aes(x=roundAge, y=Likelihood.to.recommend) +    
  geom_boxplot()

#look at NPS by age
survey %>% 
  group_by(roundAge) %>%
  summarize(nps = CalcNPS(Likelihood.to.recommend))%>%
  ggplot() +
    aes(x=roundAge, y=nps) +
    geom_col()

#a more detailed look at NPS by age
survey %>% 
  group_by(Big.Arrival.Delay, roundAge) %>%
  summarize(nps=CalcNPS(Likelihood.to.recommend)) %>%
  ggplot() +
    aes(x= roundAge, y=nps, 
      fill= Big.Arrival.Delay) + 
    geom_col(position = "dodge")


survey %>% 
  ggplot() + 
  geom_histogram(binwidth=1, 
                 fill="black", color="white",
                 aes(x=Likelihood.to.recommend)) + 
  facet_wrap(~Type.of.Travel)

