# Chapter 9 of Introduction to Data Science for Business
#
library(ggplot2)
library(ggmap)
us <- map_data("state")

ggplot(us)  +
  geom_point(aes(x=long,y=lat))

ggplot(us)  +
  geom_point(aes(x=long,y=lat)) + coord_map()

#zoom into NY
NYData <- us %>% filter(region=="new york")

ggplot(NYData)  +
  geom_point(aes(x=long,y=lat)) + coord_map()


#look at one group
NYGroupLevelData <- us %>% filter(group==35)
ggplot(NYGroupLevelData)  +
  geom_point(aes(x=long,y=lat)) +
  coord_map()

#generate the map using lines
ggplot(us) +
  geom_polygon(color="black", fill="white", 
               aes(x=long,y=lat, group=group)) + 
  coord_map()


#Use the readCensus function defined in Chapter 5
url <- "http://www2.census.gov/programs-surveys/popest/tables/2010-2011/state/totals/nst-est2011-01.csv"
dfStates <- readCensus(url)

#make sure everything is lowercase
dfStates$state <- tolower(dfStates$stateName)

#combine dataframes, using state and region to merge
dfStatesWithGeom<- merge(dfStates, us,   
                         by.x="state",by.y="region")

#don’t lose the order for the points in the polygon
dfStatesWithGeom<- dfStatesWithGeom%>% arrange(order)

#render the map
myMap <- ggplot(dfStatesWithGeom) +
  geom_polygon(color="black", 
               aes(x=long,y=lat, group=group, fill=july11pop)) + 
  coord_map()
myMap

#show a point
myMap + geom_point(aes(x = -100, y = 30))

#use two libraries 
library(jsonlite)
library(stringr)

#address to be geocoded
address <- "syracuse, NY"

#replace spaces with '%20' (web standard for a space)
address <- str_replace_all(address, " ", "%20")

#define the url to get the geocode
urlofGEOCode <- paste0( 
  'https://nominatim.openstreetmap.org/search/', 
  address,'?format=json&addressdetails=0&limit=1')

#get the location data, in JSON format 
df <- jsonlite::fromJSON(urlofGEOCode)

#get the longitude and latitude from the dataframe   
longitude = as.numeric(df$lon)
latitude = as.numeric(df$lat)

#show as a white point (the longitude and latitude)
myMap + geom_point(aes(x = longitude, y = latitude), 
                   color="white")



#create a dataframe with state centers and population
stateCenter <- data.frame( 
  state=tolower(state.name), 
  x=state.center$x,  
  y=state.center$y)

#merge with the the dataframe we have been using
dfStatesWithCenter <- merge(dfStatesWithGeom,  
                            stateCenter, by="state")

#don’t forget to sort on the order attribute
dfStatesWithCenter <- dfStatesWithCenter %>%
  arrange(order)	

#add a points layer with color=population
ggplot(dfStatesWithCenter) +
  geom_polygon(color="black", fill="white", 
               aes(x=long,y=lat, group=group)) +
  geom_point(aes(x=x,y=y, color=july11pop)) + 
  coord_map()


#put a map image behind the visualizaiton
bb <- c(left = min(dfStatesWithCenter$long), 
        bottom = min(dfStatesWithCenter$lat), 
        right = max(dfStatesWithCenter$long), 
        top = max(dfStatesWithCenter$lat))

#actually get the map 
#  note if the zoom is too large, it will take a long 
#  time to load the maps across the internet 
map <- get_stamenmap(bbox = bb, zoom=4)

#show the map using the ggmap command
ggmap(map)


#add a points layer with color=population
ggmap(map) +
  geom_polygon(data= dfStatesWithCenter,  
               color="black", fill="NA", 
               aes(x=long,y=lat, group=group)) +
  geom_point(data= dfStatesWithCenter, size=4, 
             aes(x=x,y=y, color=july11pop))

ggmap(map) +
  geom_polygon(data=dfStatesWithGeom, 
               color="black", alpha=0.8, 
               aes(x=long,y=lat, group=group, fill=july11pop))

###########################
# zooming with bike data
###########################
library(RCurl)
library(jsonlite)

#get the station info, similar to Chapter 7
bikeURL <- paste0('https://gbfs.citibikenyc.com/',
                  'gbfs/en/station_status.json')

apiResult <- getURL(bikeURL)
results <- fromJSON(apiResult)
stationStatus <- results$data$stations
allBikeData <- data.frame(stations)

cols <- c('num_bikes_disabled','num_docks_disabled', 'station_id',
          'num_ebikes_available', 'num_bikes_available', 'num_docks_available')
stationStatus = stationStatus[,cols]


#now get the location information for the stations
stationLocations <-
  paste0('https://gbfs.citibikenyc.com/',
         'gbfs/en/station_information.json')

apiResult <- getURL(stationLocations)
results <- fromJSON(apiResult)
stationInfo <- results$data$stations

stationInfo = stationInfo[,c('station_id','capacity', 'lon', 'lat', 'name')]

#merge the two 
allBikeData = merge(stationStatus, 
                    stationInfo, on='station_id')



#get the bounding box for the map
bb <- c(left = min(allBikeData$lon), 
        bottom = min(allBikeData$lat), 
        right = max(allBikeData$lon), 
        top = max(allBikeData$lat))

#get the new background map
mapNY <- get_stamenmap(bbox = bb, zoom=12)

# visualize the points & map, scaling the points
ggmap(mapNY) +  
  geom_point(data=allBikeData, 
             alpha=0.5, color="black",
             aes( x=lon,
                  y=lat,
                  size=num_bikes_available)) + 
  scale_size(range=c(0, 2))



###########################
# case
###########################
#get the US map
us <- map_data("state")

#Calculate the bounding box
bb <- c(left = min(us$long), bottom = min(us$lat), 
        right = max(us$long), top = max(us$lat))
map <- get_stamenmap(bbox = bb, zoom=5)

#generate a dataframe with NPS values per city
NPSbyCity  <- survey %>% 
  group_by(Origin.City) %>% 
  summarize(cityNPS=CalcNPS(Likelihood.to.recommend),
            num=n(), olong=mean(olong), olat=mean(olat))

ggmap(map) +
  geom_point(data=NPSbyCity, 
             aes(x=olong, y=olat,  size=num, color=cityNPS))


#explore per state NPS
npsByStateDF <- survey %>% 
  filter(Origin.State != "Puerto Rico") %>%
  mutate(state=tolower(Origin.State)) %>%
  group_by(state) %>%
  summarize(stateNPS = 
              CalcNPS(Likelihood.to.recommend))

#merge the geom state info with the data
NPSwithGeom <- merge(us, npsByStateDF, 
                     by.y="state", by.x="region")

#make sure the points are in the correct order
NPSwithGeom <- NPSwithGeom %>% arrange(order)

#visualize the "heatmap"
ggmap(map) +
  geom_polygon(data= NPSwithGeom, 
               color="black", alpha=0.9, 
               aes(x=long,y=lat, group=group, fill=stateNPS))

ggmap(map) +
  geom_polygon(data= NPSwithGeom, 
               color="black", alpha=0.9, 
               aes(x=long,y=lat, group=group, fill=stateNPS)) +
  geom_point(data=NPSbyCity, 
             aes(x=olong, y=olat, color=cityNPS)) + 
  scale_color_gradient(low="black", high="white")



#########################################
# Create the first figure in this chapter
#########################################
library("maps")
library("mapproj")

savemar <- par("mar")
par(mfrow=c(1,2))

par(mar=c(1,1,1,1))
pts <- map("world", proj="gilbert", wrap=c(-180,180))
map.grid(c(-180,180,-90,90),nx=7,ny=3,pretty=FALSE,label=TRUE,lty=2, cex=0.75)

pts <- map("world", proj="gilbert", wrap=c(-180,180))
map.grid(c(-180,180,-90,90),nx=3,ny=7,pretty=FALSE,label=TRUE,lty=2, cex=0.75)

par(mfrow=c(1,1))
par(mar=savemar)


# Compare raster to vector for the figure
plot(NULL, xlim=c(0,40), ylim=c(0,40), ylab="", xlab="")
image <- matrix(1, ncol = 40, nrow = 40)
diag(image) <- 0
image <- as.raster(image)
rasterImage(image, xleft=0, ybottom=0, xright=40, ytop=40, angle=0, interpolate=F)
abline(0,1, lwd=5)


